@@ -163,8 +163,10 @@ public:
 
     void setup(QCA::Direction dir,
 	       const QCA::SymmetricKey &key,
-	       const QCA::InitializationVector &iv)
+	       const QCA::InitializationVector &iv,
+	       const QCA::AuthTag &tag)
     {
+	Q_UNUSED(tag);
 	m_direction = dir;
 	err =  gcry_cipher_open( &context, m_cryptoAlgorithm, m_mode, 0 );
 	check_error( "gcry_cipher_open", err );
@@ -195,6 +197,12 @@ public:
 	return blockSize;
     }
 
+    QCA::AuthTag tag() const
+    {
+    // For future implementation
+	return QCA::AuthTag();
+    }
+
     bool update(const QCA::SecureArray &in, QCA::SecureArray *out)
     {
         QCA::SecureArray result( in.size() );
